/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.platform.controller;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springblade.common.base.BaseEntity;
import org.springblade.common.enums.YesOrNoEnum;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.groupon.entity.SelfTake;
import org.springblade.modules.groupon.entity.UserSelfTake;
import org.springblade.modules.groupon.service.ISelfTakeService;
import org.springblade.modules.groupon.service.IUserSelfTakeService;
import org.springblade.modules.platform.service.UserAppService;
import org.springblade.modules.system.entity.User;
import org.springblade.modules.system.entity.UserApp;
import org.springblade.modules.system.service.IUserService;
import org.springblade.modules.system.service.impl.UserServiceImpl;
import org.springblade.modules.system.vo.UserAppVO;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 用户列表 控制器
 *
 * @author BladeX
 * @since 2022-03-28
 */
@RestController
@RequestMapping("platform/userApp")
@Api(value = "用户表", tags = "用户表接口")
public class UserAppController extends BladeController {

	@Resource
	private UserAppService userAppService;

	@Resource
	private IUserService userService;

	@Resource
	private IUserSelfTakeService userSelfTakeService;
	@Resource
	private ISelfTakeService selfTakeService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入userApp")
	public R<UserApp> detail(UserApp userApp) {
		UserApp detail = userApp.selectOne(Condition.getQueryWrapper(userApp));
		return R.data(detail);
	}

	/**
	 * 分页 app用户表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入userApp")
	public R<IPage<UserAppVO>> list(UserAppVO vo, Query query) {
		IPage<UserAppVO> pages = userAppService.pageList(Condition.getPage(query), vo);
		return R.data(pages);
	}


	/**
	 * 新增 app用户表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入userApp")
	public R save(@Valid @RequestBody UserApp userApp) {
		return R.status(userApp.insert());
	}

	/**
	 * 修改 app用户表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入userApp")
	public R update(@Valid @RequestBody UserApp userApp) {
		return R.status(userApp.updateById());
	}

	/**
	 * 新增或修改 app用户表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入userApp")
	public R submit(@Valid @RequestBody UserAppVO userApp) {
		User user = userService.getById(userApp.getUserId());
		user.setName(userApp.getName());
		user.setSex(userApp.getSex());
		user.setRealName(userApp.getRealName());
		userService.updateById(user);
		return R.status(userApp.insertOrUpdate());
	}
	

	/**
	 * 删除 app用户表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		UserApp userApp = new UserApp();
		List<Long> list = Func.toLongList(ids);
		list.forEach(userApp::deleteById);
		return R.status(true);
	}

	@Transactional(rollbackFor = Exception.class)
	@PostMapping("/set-up-commander")
	@ApiOperationSupport(order = 9)
	@ApiOperation(value = "设为团长", notes = "传入userApp")
	public R setUpCommander(@RequestBody UserAppVO userApp) {
		Assert.notNull(userApp.getSelfTakeId(), "自提点ID不能为空");
		// 修改用户团长标识
		UserApp user = userAppService.getByUserId(userApp.getUserId());
		Assert.notNull(user, "数据不存在");
		if (YesOrNoEnum.YES.getValue().equals(user.getLivingHall())) {
			return fail("该用户已成为团长");
		}
		user.setLivingHall(YesOrNoEnum.YES.getValue());
		userAppService.updateById(user);

		// 添加用户自提点信息
		SelfTake selfTake = selfTakeService.getById(userApp.getSelfTakeId());
		Assert.notNull(selfTake, "自提点数据不存在");
		UserSelfTake userSelfTake = new UserSelfTake();
		userSelfTake.setSelfTakeId(userApp.getSelfTakeId());
		userSelfTake.setUserId(user.getUserId());
		userSelfTakeService.save(userSelfTake);


		User user1 = userService.getById(user.getUserId());
		if (StrUtil.isBlank(user1.getPassword())) {
			// 设置团长后台账号(重置密码)
			userService.resetPassword(userApp.getUserId().toString());
		}
		// 设置团长角色
		userService.setRoleByRoleName(user.getUserId(), "团长");
		// 设置自提点电话
		if (StrUtil.isBlank(selfTake.getContactNumber())) {
			selfTake.setContactNumber(user1.getPhone());
			selfTake.setContactPerson(user1.getRealName());
			selfTakeService.updateById(selfTake);
		}
		return success("ok");
	}

	@PostMapping("/remove-commander")
	@ApiOperationSupport(order = 10)
	@ApiOperation(value = "移除团长", notes = "传入userApp")
	public R removeCommander(@RequestBody UserAppVO userApp) {
		// 修改用户团长标识
		UserApp user = userAppService.getByUserId(userApp.getUserId());
		Assert.notNull(user, "数据不存在");
		user.setLivingHall(YesOrNoEnum.NO.getValue());
		userAppService.updateById(user);

		// 移除绑定的自提点数据
		List<UserSelfTake> list = userSelfTakeService.lambdaQuery().eq(UserSelfTake::getUserId, user.getUserId()).list();
		List<Long> collect = list.stream().map(BaseEntity::getId).collect(Collectors.toList());
		userSelfTakeService.removeByIds(collect);

		// 解绑团长角色
		userService.removeRoleByRoleName(user.getUserId(), "团长");

		return R.status(true);
	}


}
